---
title: DMX512
breadcrumbs:
- title: Media
- title: Lighting
---
{% include header.md %}

## General

- Used for controlling stage equipment like RGB(WA) lights and stuff.
- "DMX512" means digital multiplex with 512 addresses.
- RDM (remote device management) is a separate protocol used to remotely configure "menu settings" (like the DMX address) on devices.
- Art-Net is a separate protocol used to transfer DMX512 and RDM over an Ethernet network. (sACN is similar.)

## Protocol

- A unidirectional, master-slave, bus protocol, with a single controller as the master and one or more units/devices/fixtures as slaves (master-slave terminology still used here).
- A closed group of a controller and some units is called a "universe". Each universe consists of 512 channels (aka addresses or slots) (where 1-511 are usable). Some mixers support multiple universes, but all must be electrically disjunct and running on separate cables.
- Each unit uses one or more consecutive channels, starting at the pre-configured channel/address.
- Each channel takes a value between 0 and 255 (8-bit), representing e.g. an color component value.
- Electrically, it uses the RS-485 serial protocol.
- RS-485 is designed to be a balanced (or differential) system using twisted pairs, to reduce both emitted EMI and received EMI (after merging the signals to get rid of common-mode noise). In DMX this is supported by XLR5 but not XLR3 connectors.
- RS-485 does not use any kind of error checking and correction.

## Cabling and Programming

- Typically using XLR5 (compliant) or XLR3 (commonly used but non-compliant) connectors.
- Units are addressed using a pre-configured address. Multiple units may use the same address if they should be controlled using the same address and are roughly channel-compatible.
- When connecting multiple units to the same controller, the units must be daisy chained together (unless using some fancy splitter). A maximum of 32 units may be chained (according to the specification). Units typically have one input and one output for proper daisy chaining (even if they have multiple outputs, you should only use one).
- Long lines (sum of all cables in the daisy chain, about 150-300m) should generally always be terminated using a DMX terminator for best possible signal integrity. For short runs it will often work without a terminator (until it won't). This is in order to reduce reflected signals (inter-symbol interference from signals bouncing back from the end) and noise. The terminator is a connector containing a 120ohm resistor (depending on the characteristic impedance of the cable, but typically 120ohm).
- Although both audio and DMX (may) use cables with XLR3 connectors (DMX should use XLR5, though), DMX should use cables specific to DMX to meet the DMX/RS-485 cable requirements (i.e. not mic cables).
- For less expensive cabling for permanent setups, Cat5e or Cat6 may be used instead, with 1-4 universes per cable. Not that it should be terminated according to ESTA's DMX512-over-Cat5 specification, see [this](https://support.etcconnect.com/ETC/FAQ/DMX_Over_Cat5).
- Using proper cabling, it may sometimes achieve data rates over 10Mb/s and distances over 1000m.
- To run multiple DMX universes over a large area, Art-Net (or sACN) may be used to run DMX over Ethernet.

## RDM

- "RDM" means remote device management and is a protocol used to remotely configure "menu settings" (like the DMX address) on devices.
- This is extremely useful both for fixing settings on inaccessible devices, as well as making it generally more practical to configure devices.
- Electrically, RDM is run on the same cable as DMX512 and transmits data between DMX signals/commands on the same data wires.
- All of the controller, devices and eventual splitters/repeaters need to support RDM in order for it to work.
- Devices can be easily discovered by the controller by running an RDM scan.
- Devices not RDM-compliant may exhibit issues like flickering when RDM traffic exists on the line.

## Resources

- [RS-485 Cable Specification Guide (Maxim Integrated)](https://www.maximintegrated.com/en/design/technical-documents/tutorials/7/763.html)
- [DMX Over Cat5 (Electronic Theatre Controls Inc)](https://support.etcconnect.com/ETC/FAQ/DMX_Over_Cat5)

{% include footer.md %}
